use bevy::app::PluginGroupBuilder;
use bevy::log::{Level, LogPlugin};
use bevy::prelude::*;
use bevy::window::{PresentMode, WindowMode, WindowResolution};
use bevy_embedded_assets::EmbeddedAssetPlugin;

use crate::system::load_config::{get_asset_path_string, initial_size};

pub struct DefaultResourcesPlugin;
impl Plugin for DefaultResourcesPlugin {
	fn build(&self, app: &mut App) {
		app.insert_resource(Msaa::Off)
			.insert_resource(ClearColor(Color::hex("040720").unwrap()));
	}
}
pub fn configure_default_plugins() -> PluginGroupBuilder {
	let (width, height) = initial_size();
	let mut plugin_set = DefaultPlugins
		.set(WindowPlugin {
			primary_window: Some(Window {
				resolution: WindowResolution::new(width, height),
				resizable: true,
				mode: WindowMode::Windowed,
				title: String::from("Game Core"),
				present_mode: PresentMode::AutoNoVsync,
				fit_canvas_to_parent: true,
				..Default::default()
			}),
			..Default::default()
		})
		.set(AssetPlugin {
			asset_folder: get_asset_path_string(),
			watch_for_changes: true,
		})
		.set(ImagePlugin::default_nearest())
		.set(LogPlugin {
			filter: String::from(
				"info,game_core=debug,symphonia_core=warn,symphonia_format_ogg=warn,winit=warn,symphonia_bundle_mp3=warn,wgpu_core=warn,wgpu_hal=warn",
			),
			level: Level::DEBUG,
		});

	if cfg!(feature = "embed") {
		plugin_set.add_before::<AssetPlugin, _>(EmbeddedAssetPlugin)
	} else {
		plugin_set
	}
}

pub struct InitAppPlugins;
impl PluginGroup for InitAppPlugins {
	fn build(self) -> PluginGroupBuilder {
		configure_default_plugins()
			.add(DefaultResourcesPlugin)
			.add(super::web::WebPlugin)
	}
}
